;================================
;          ZX-Assembler-2 
;      by Artic Computing LTD
;================================
; 	    Dancers
;      Greg Harder 2020
;================================

ORG 4084

@Keyboard=02BB			;Keyboard scanning subroutine
@Decode=07BD			;Keyboard decode subroutine
@DFile=400C			;Address of display file
@LastK=4025			;Keyboard debounce value
@Frames=4034			;16436, Frame counter
@BKG=403D			;16445, Background character
@Dancers=4048			;16461, Dancers data

@Dancers
	CALL Release		;Wait for key release
	LD HL,(DFile)		;Get address of line 23
	LD BC,02F8		;Offset to line 23
	ADD HL,BC
	LD (Print+2),HL		;Set line 23 address
	LD A,02			;Initial dancers count
	LD (Dancers),A		;Set dancers count to 2
	CALL CLS		;CLS with (BKG) chr
	LD A,D			;A=(BKG) chr
	LD (ClearPit+3),A	;Set erase value
	LD (MoshPit+2),A	;Set BonW or WonB value
	LD BC,Dancers+1		;Fill initial Xcoords
@DancerData
	CALL RND8Bit
	AND 1F			;Range=0 to 31
	INC C
	LD (BC),A		;DancerX Xcoord
	INC C
	LD A,5D
	CP C
	JR NZ DancerData	;Loop until start of Membot found
@DanceLoop
	LD HL,Dancers
	LD B,(HL)		;Get number of Dancers
	INC HL			;Dancer address
@MoshPit
	LD A,(HL)		;DancerX Chr
	ADD A,00		;00 or 128, POKEd value from above
	INC HL			;DancerX Xcoord
	LD D,00			;Set Ycoord
	LD E,(HL)		;Set Xcoord	
	CALL Print		;PRINT AT D,E
	INC HL			;Next dancer address
	DJNZ MoshPit		;Loop until B=0
;Scroll Up
	LD HL,(DFile)
	PUSH HL 
	LD DE,0021 
	ADD HL,DE 
	POP DE 
	LD BC,02F7 
	LDIR
	LD HL,Dancers		;Clear the floor
	LD B,(HL)		;Get dancers count
@ClearPit
	INC HL			;DancerX Chr
	INC HL			;DancerX Xcoord
	LD A,00			;00 or 128, POKEd value from above
	LD D,00			;Set Ycoord
	LD E,(HL)		;Set Xcoord
	CALL Print		;PRINT AT D,E
	PUSH HL
	CALL RND8Bit		;Get random number
	POP HL
	AND 01			;Odd or even?
	LD A,(HL)		;A=Xcoord
	JR Z StepLeft		;Jump if even
;StepRite
	INC A			;Step right
	CP +32			;Right edge?
	JR C SetStep
;	XOR A			;Wrap left
	DEC A
	DEC A
	JR SetStep
@StepLeft
	DEC A			;Step left
	CP FF
        JR NZ SetStep
;	LD A,+31		;Wrap right
	INC A
	INC A
@SetStep
	LD (HL),A		;Set new Xcoord
	DJNZ ClearPit
	CALL Keysort		;Sort out any key presses
@Delay
	LD BC,0801	 	;Delay Loop
	DEC BC 
	LD A,B 
	OR C 
	JR NZ -5
	JR DanceLoop		;Loop until key "Q"uit pressed
;================================
;PRINT AT 23,BC;A
;================================
@Print
	PUSH HL
	LD HL,0000		;DFile line 23 addr, POKEd from above
	ADD HL,DE		;Add Xcoord offset
	LD (HL),A		;Print A
	POP HL
	RET
;================================
; Test for keypress.
; On exit...
;  A=key chr code
;================================
@KeyScan
	CALL Keyboard		;Call keyboard scanning routine
	LD B,H 			;BC=Raw Key values
	LD C,L 
	LD D,C			;Test for keypress 
	INC D 
	LD A,01 
	RET Z 			;No keypress
	CALL Decode		;Call keyboard decode routine, BC to ZX81 codes
	LD A,(HL)		;A=CHR CODE
	RET
;================================
; Sort out the key presses
;================================
@KeySort
	CALL Keyscan
	CP 01			;Test for keypress
	RET Z			;Return if nokey
	CP "0"			;Key <="."?
	JR C Release		;Yes, return via release
	CP "A"			;Key <="9"?
	JR NC +11		;Jump if A>="A"
	SUB 1C			;Convert keycode to digit value
	AND A			;A=0?
	JR NZ +2		;Jump if not 0
	LD A,0A			;Set A=10
	LD (Dancers),A		;Set dancers count
	RET
	LD HL,Delay+2		;Delay counter addr
;KeyS
	CP "S"			;"S"lower?
	JR NZ +2		;No, move on
	INC (HL)		;Increase delay count
	RET
;KeyF
	CP "F"			;"F"aster?
	JR NZ +7		;No, move on
	LD A,(HL)		;Test for 0
	CP 01
	JR Z +1
	DEC (HL)		;Decrease delay count
	RET
;KeyI
	CP "I"			;"I"reg cycle?
	JR NZ +16		;No, move
@IReg
	LD A,FE			;Current Ireg value
	INC A			;Add 2
	INC A
	CP 20			;32?
	JR NZ +1		;No, move on
	XOR A			;Yes, reset to 0
	LD (Ireg+1),A		;Store Ireg
	LD I,A			;Set Ireg
	JR Release		;Return via release
;KeyP
	CP "P"			;"P"ixels
	LD C,+110		;GOTO 110
	JR Z BASRet
;KeyB
	CP "B"			;"B"locks
	LD C,+130		;GOTO 130
	JR Z BASRet
;KeyQ
	CP "Q"			;"Q"uit?
	LD C,+200		;GOTO 200
	JR Z BASREt
	LD C,+100		;GOTO 100
;KeyW
	CP "W"			;"W"hite/Black (BKG) switch
	JR NZ +14
	LD A,(BKG)		;Get current (BKG)
	ADD A,80		;New BKG value
	LD (BKG),A		;Set (BKG)
	JR BASRet		;GOTO 100
;KeyR
	CP "R"			;"R"estart?
	JR NZ Release		;No, return via release
@BASRet
	LD B,00			;B=Hi byte of line number
	POP DE			;Clear stack
	LD A,1E			;Reset RegI
        LD I,A
	RET			;Return GOTO BC to BASIC
;================================
; Wait for key release
;================================
@Release
	LD A,(LastK)		;Test for keypress
	CP FF			;Keypressed?
	JR NZ Release		;Yes, loop back
	RET
;================================
; Clear screen with chr A
;================================
@CLS
	LD D,(IY+61)		;CLS with BKG chr
	LD HL,(DFile)
	LD BC,0318		;Byte count
	INC HL
	BIT 6,(HL)		;N/L?
	JR NZ +1		;Yes, skip it
	LD (HL),D		;Print BKG chr.
	DEC BC 			;Decrease counter
	LD A,B 
	OR C 
	JR NZ -11		;Loop until BC=0
	RET
;================================
; 8-Bit Random Number Generator
; Reg BC not used
; On exit...
;  Reg A=random number in the
;        range 0 to 255
;================================
@Rnd8Bit
	LD DE,0000		;Seed
	LD HL,(Frames)     	;Create pointer to ROM address
	DEC DE	
	ADD HL,DE
	LD A,H
	AND 1F
	LD H,A			;Limit address to <=8192
	LD (Rnd8Bit+1),HL	;New seed
	LD A,(HL)		;Get value at ROM address
	RET